
use strict;
my $mode = 'parse';

my $prog = 'run.pl';


my @instances = qw(
dsj1000.tsp
d1291.tsp
d1655.tsp
fl1400.tsp
fl1577.tsp
nrw1379.tsp
pcb1173.tsp
pr1002.tsp
rl1304.tsp
rl1323.tsp
rl1889.tsp
si1032.tsp
u1060.tsp
vm1084.tsp
u1432.tsp
u1817.tsp
vm1748.tsp
d2103.tsp
u2152.tsp
u2319.tsp
pr2392.tsp
pcb3038.tsp
fnl4461.tsp
d15112.tsp
rl5915.tsp
rl5934.tsp
pla7397.tsp

E1k.0
E1k.1
E1k.2
E1k.3
E1k.4
E1k.5
E1k.6
E1k.7
E1k.8
E1k.9
E3k.0
E3k.1
E3k.2
E3k.3
E3k.4
E10k.0
E10k.1
E10k.2
E31k.0
E31k.1

C1k.0
C1k.1
C1k.2
C1k.3
C1k.4
C1k.5
C1k.6
C1k.7
C1k.8
C1k.9
C3k.0
C3k.1
C3k.2
C3k.3
C3k.4
C10k.0
C10k.1
C10k.2
C31k.0
C31k.1

fl3795.tsp
d18512.tsp
usa13509.tsp
brd14051.tsp
rl11849.tsp
pla33810.tsp
pla85900.tsp

M1k.0
M1k.1
M1k.2
M1k.3
M3k.0
M3k.1

E100k.0
E100k.1
E316k.0
E1M.0
E3M.0
C100k.0
C100k.1
C316k.0
M10k.0
);

sub name_to_size {
   my ($name) = @_;
   $name =~ s/^[A-Za-z]+//;  # leading chars
   $name =~ s/\..*//;  # after .
   $name =~ m/(\d+)(\D+)/;
   my ($num,$suffix) = (int($1),$2);

   if ( $suffix eq 'k' ) {
      $num *= 1000;
   } elsif ( $suffix eq 'M' ) {
      $num *= 1000000;
   } elsif ( $suffix eq '' ) {
      # ok
   } else {
      die "$prog: name_to_size failed for $name\n";
   }
   return $num;
}

my %instance_classification = qw(
dsj1000.tsp    tsplib
d1291.tsp    tsplib
d1655.tsp    tsplib
fl1400.tsp    tsplib
fl1577.tsp    tsplib
nrw1379.tsp    tsplib
pcb1173.tsp    tsplib
pr1002.tsp    tsplib
rl1304.tsp    tsplib
rl1323.tsp    tsplib
rl1889.tsp    tsplib
si1032.tsp    tsplib
u1060.tsp    tsplib
vm1084.tsp    tsplib
u1432.tsp    tsplib
u1817.tsp    tsplib
vm1748.tsp    tsplib
d2103.tsp    tsplib
u2152.tsp    tsplib
u2319.tsp    tsplib
pr2392.tsp    tsplib
pcb3038.tsp    tsplib
fnl4461.tsp    tsplib
d15112.tsp    tsplib
rl5915.tsp    tsplib
rl5934.tsp    tsplib
pla7397.tsp    tsplib

E1k.0  e
E1k.1  e
E1k.2  e
E1k.3  e
E1k.4  e
E1k.5  e
E1k.6  e
E1k.7  e
E1k.8  e
E1k.9  e
E3k.0  e
E3k.1  e
E3k.2  e
E3k.3  e
E3k.4  e
E10k.0  e
E10k.1  e
E10k.2  e
E31k.0  e
E31k.1  e

C1k.0 c
C1k.1 c
C1k.2 c
C1k.3 c
C1k.4 c
C1k.5 c
C1k.6 c
C1k.7 c
C1k.8 c
C1k.9 c
C3k.0 c
C3k.1 c
C3k.2 c
C3k.3 c
C3k.4 c
C10k.0 c
C10k.1 c
C10k.2 c
C31k.0 c
C31k.1 c

fl3795.tsp    tsplib
d18512.tsp    tsplib
usa13509.tsp    tsplib
brd14051.tsp    tsplib
rl11849.tsp    tsplib
pla33810.tsp    tsplib
pla85900.tsp    tsplib

M1k.0 m
M1k.1 m
M1k.2 m
M1k.3 m
M3k.0 m
M3k.1 m

E100k.0  e
E100k.1  e
E316k.0  e
E1M.0  e
E3M.0  e
C100k.0 c
C100k.1 c
C316k.0 c
M10k.0 m
);


#@instances = ($instances[0..2]);

my %bound_held_karp = ();

open BOUNDS, "<bounds.txt" or die "$prog: Can't open bounds.txt for read: $!";
while (<BOUNDS>) {
   my $line = $_;
   if ( $line =~ m/^(\S+)\s+(\S+)(.*)/ ) {
      my ($instance, $hk) = ($1,$2);
      $bound_held_karp{$instance} = $hk+0;
   }
}

# first key $candidate, 
# second key is $type, 
# third key is $instance, 
# fourth key is seed,
# value is array ref of [ pct, total time, lk time ]
my %results = ();

my $num_parse_errors= 0;
foreach my $candidate ( ("nn 20", "nn 20 or nq 5") ) {
   foreach my $type ( qw( cc base ) ) {
      my $lk = "mingw_lk_$type";
      my $cname = $candidate; $cname=~s/ /_/g;
      my $dir = $lk."_out_$cname";
      mkdir $dir unless -d $dir;
      if ( $mode ne 'parse' ) {
         close STDERR;
         open STDERR, ">>$dir/errs.log";
         close STDOUT;
         open STDOUT, ">>$dir/out.log";
      }
      foreach my $seed ( 2002..2006 ) {
         mkdir "$dir/$seed" unless -d "$dir/$seed";

         foreach my $instance (@instances) {
            my $out_file = "$dir/$seed/$instance.out";
            my $working_out_file = "$dir/$seed/$instance.working";
            my $instance_with_no_extension = $instance;
            $instance_with_no_extension =~ s/\.tsp$//;

            my $hk = $bound_held_karp{$instance} 
                     || $bound_held_karp{$instance_with_no_extension};

            (defined $hk && $hk > 0) or die "$prog: $instance has no held karp bound";

            my $cmd = "$lk -c $candidate --maxdepth 50 --seed $seed"
                     ." -r two-level -l held_karp $hk -v $instance";

            if ( $mode eq 'parse' ) {
               #print "parsing $out_file\n";
               my ($status,$pct,$total_t,$lk_t) = get_result_from_file($out_file);
               if ( $status ) {
                  $results{$candidate} = {} 
                     unless defined $results{$candidate};
                  $results{$candidate}{$type} = {} 
                     unless defined $results{$candidate}{$type};
                  $results{$candidate}{$type}{$seed} = {} 
                     unless defined $results{$candidate}{$type}{$seed};
                  $results{$candidate}{$type}{$seed}{$instance}
                     = [ $pct, $total_t, $lk_t ];
                  #print "   $pct $total_t $lk_t\n";

               } else {
                  warn "Warning: Bad parse of $out_file: $pct\n";
                  $num_parse_errors++;
               }
            } else {
               unless ( -f $out_file ) {
                  $cmd .= " >$working_out_file";
                  print $cmd,"\n";
                  system ($cmd);
                  rename $working_out_file, $out_file
                     or die "$prog: $instance can't rename working file to $out_file: $!";
               }
            }
         }
      }
   }
}


if ( $mode eq 'parse' ) {
   foreach my $class ( qw( tsplib e c m ) ) {
      print ".....class $class\n";
      foreach my $candidate ( ("nn 20", "nn 20 or nq 5") ) {
         my $cstr = $candidate; $cstr=~s/\s//g;
         my $pct_file = "parse.$class.$cstr.pct.tex";
         my $tot_file = "parse.$class.$cstr.tot.tex";
         my $lk_file = "parse.$class.$cstr.lk.tex";

         open PCT_FILE, ">$pct_file" or die "Can't open $pct_file for write: $!";
         open TOT_FILE, ">$tot_file" or die "Can't open $tot_file for write: $!";
         open LK_FILE, ">$lk_file" or die "Can't open $lk_file for write: $!";

         my @sum_of_logs_ratio = ();
         my @count_ratio = ();
         my @sum_of_logs_base = ();
         my @count_base = ();
         my @sum_of_logs_cc = ();
         my @count_cc = ();

         foreach my $instance (sort { name_to_size($a) <=> name_to_size($b) } @instances) {
            next unless $instance_classification{$instance} eq $class;
            my %i_result = ();
            foreach my $type ( qw( cc base ) ) {
               my ($avg_pct, $avg_tot, $avg_lk) = (0,0,0);
               my $count = 0;
               foreach my $seed ( 2002..2006 ) {
                  my $aref = $results{$candidate}{$type}{$seed}{$instance};
                  my $pct = $$aref[0]+0;
                  my $tot = $$aref[1]+0;
                  my $lk = $$aref[2]+0;
                  print join(' ', $seed, $instance, $pct,$tot,$lk),"\n";
                  $avg_pct += $pct;
                  $avg_tot += $tot;
                  $avg_lk += $lk;
                  $count++ if $tot > 0;
               }
               if ( $count > 0 ) {
                  $avg_pct /= $count;
                  $avg_tot /= $count;
                  $avg_lk /= $count;
                  print "$class avg $type ", 
                        join(' ', $instance, $avg_pct,$avg_tot,$avg_lk),"\n";
                  $i_result{$type} = [ $avg_pct, $avg_tot, $avg_lk, $count ];
               }
            }
            next unless defined $i_result{'cc'} && defined $i_result{'base'};

            foreach my $i (0..2) {
               $sum_of_logs_ratio[$i] += log(
                   (${$i_result{'cc'}}[$i]/${$i_result{'base'}}[$i]) );
               $count_ratio[$i] ++;
               $sum_of_logs_base[$i] += log(${$i_result{'base'}}[$i]);
               $count_base[$i] ++;
               $sum_of_logs_cc[$i] += log(${$i_result{'cc'}}[$i]);
               $count_cc[$i] ++;
            }
            printf "$class $candidate $instance "
                   ."base %.2f %.1f %.1f cc %.2f %.1f %.1f "
                   ."compare %.2f %.2f %.2f\n",
                   @{$i_result{'base'}},
                   @{$i_result{'cc'}},
                   (${$i_result{'cc'}}[0]-${$i_result{'base'}}[0]),
                   (${$i_result{'cc'}}[1]/${$i_result{'base'}}[1]),
                   (${$i_result{'cc'}}[2]/${$i_result{'base'}}[2]);
            printf PCT_FILE " $instance & %.2f & %.2f & %.2f \\\\ \\hline \%\% \%d \%d \n", 
                   ${$i_result{'base'}}[0],
                   ${$i_result{'cc'}}[0],
                   ${$i_result{'cc'}}[0]/${$i_result{'base'}}[0],
                   ${$i_result{'cc'}}[3],
                   ${$i_result{'base'}}[3];
            printf TOT_FILE " $instance & %.2f & %.2f & %.2f \\\\ \\hline\%\% \%d \%d \n", 
                   ${$i_result{'base'}}[1],
                   ${$i_result{'cc'}}[1],
                   ${$i_result{'cc'}}[1]/${$i_result{'base'}}[1],
                   ${$i_result{'cc'}}[3],
                   ${$i_result{'base'}}[3];
            printf LK_FILE " $instance & %.2f & %.2f & %.2f \\\\ \\hline \%\% \%d \%d \n", 
                   ${$i_result{'base'}}[2],
                   ${$i_result{'cc'}}[2],
                   ${$i_result{'cc'}}[2]/${$i_result{'base'}}[2],
                   ${$i_result{'cc'}}[3],
                   ${$i_result{'base'}}[3];
         }
         my $fmt_string =
           '\\hline Geometric average & %.2f & %.2f & %.2f \\\\ \\hline'."\n";
         if ( $count_base[0] > 0 && $count_base[1] > 0 && $count_base[2] > 0 ) {
            printf PCT_FILE $fmt_string,
               exp( $sum_of_logs_base[0]/$count_base[0] ),
               exp( $sum_of_logs_cc[0]/$count_cc[0] ),
               exp( $sum_of_logs_ratio[0]/$count_ratio[0] );
            printf TOT_FILE $fmt_string,
               exp( $sum_of_logs_base[1]/$count_base[1] ),
               exp( $sum_of_logs_cc[1]/$count_cc[1] ),
               exp( $sum_of_logs_ratio[1]/$count_ratio[1] );
            printf LK_FILE $fmt_string,
               exp( $sum_of_logs_base[2]/$count_base[2] ),
               exp( $sum_of_logs_cc[2]/$count_cc[2] ),
               exp( $sum_of_logs_ratio[2]/$count_ratio[2] );
         }
      }
   }
}


sub get_result_from_file {
   # Return a 4-tuple (status,pct_over_hk,total_time,lk_time)
   # If status != 0 that means ok.
   # Otherwise, the 2nd element is the error.
   my ($file) = @_;
   if ( open(OUT_FILE,"<$file" ) ) {
      my ($pct_over_hk,$total_time,$lk_time,$cc_overhead_time);
      $cc_overhead_time = 0;
      my $in_cc_overhead = 0;
      while (<OUT_FILE>) {
         my $line = $_;
         if ( $line =~ m/^Milestone:Final: length \S+\s+([^%]+)% above held/ ) {
            $pct_over_hk = $1+0;
         } elsif ( $line =~ m/^LK phase ended with incumbent_len == \S+ after (\S+) sec for LK and (\S+) sec for ds\+LK/ ) {
            ($lk_time,$total_time) = ($1+0,$2+0);
         } elsif ( $line =~ m/^Build a MST/ ) {
            $in_cc_overhead = 1;
            if ( $file =~ m/_base_/ ) {
               die "cc overhead in $file\n";
            }
         } elsif ( $line =~ m/^Build the adjacency/ ) {
            $in_cc_overhead = 0;
         } elsif ( $in_cc_overhead && $line =~ m/\s+(\d+\.?\d+) user sec/ ) {
            $cc_overhead_time += $1;
         }
      }
      close OUT_FILE;

      defined $pct_over_hk or return (0, "No HK pct");
      defined $lk_time or return (0, "No LK time");
      defined $total_time or return (0, "No total time");
      #printf "$file cc overhead %.2f\n", $cc_overhead_time;

      return (1,$pct_over_hk, $total_time, $lk_time+$cc_overhead_time );
   } else {
      return (0,"$!");
   }
}
