

my $prog = 'run.pl';
use strict;
my $dry_run = 0;


my @instances = qw(
dsj1000.tsp
d1291.tsp
d1655.tsp
fl1400.tsp
fl1577.tsp
nrw1379.tsp
pcb1173.tsp
pr1002.tsp
rl1304.tsp
rl1323.tsp
rl1889.tsp
si1032.tsp
u1060.tsp
vm1084.tsp
u1432.tsp
u1817.tsp
vm1748.tsp
d2103.tsp
u2152.tsp
u2319.tsp
pr2392.tsp
pcb3038.tsp
fnl4461.tsp
d15112.tsp
rl5915.tsp
rl5934.tsp
pla7397.tsp

E1k.0
E1k.1
E1k.2
E1k.3
E1k.4
E1k.5
E1k.6
E1k.7
E1k.8
E1k.9
E3k.0
E3k.1
E3k.2
E3k.3
E3k.4
E10k.0
E10k.1
E10k.2
E31k.0
E31k.1

C1k.0
C1k.1
C1k.2
C1k.3
C1k.4
C1k.5
C1k.6
C1k.7
C1k.8
C1k.9
C3k.0
C3k.1
C3k.2
C3k.3
C3k.4
C10k.0
C10k.1
C10k.2
C31k.0
C31k.1

fl3795.tsp
d18512.tsp
usa13509.tsp
brd14051.tsp
rl11849.tsp
pla33810.tsp
pla85900.tsp

M1k.0
M1k.1
M1k.2
M1k.3
M3k.0
M3k.1

E100k.0
E100k.1
E316k.0
E1M.0
E3M.0
C100k.0
C100k.1
C316k.0
M10k.0
);


#@instances = ($instances[0..2]);

my %bound_held_karp = ();

open BOUNDS, "<bounds.txt" or die "$prog: Can't open bounds.txt for read: $!";
while (<BOUNDS>) {
   my $line = $_;
   if ( $line =~ m/^(\S+)\s+(\S+)(.*)/ ) {
      my ($instance, $hk) = ($1,$2);
      $bound_held_karp{$instance} = $hk+0;
   }
}

foreach my $candidate ( ("nn 20", "nn 20 or nq 5") ) {
   foreach my $type ( qw( cc base ) ) {
      my $lk = "mingw_lk_$type";
      my $cname = $candidate; $cname=~s/ /_/g;
      my $dir = $lk."_out_$cname";
      mkdir $dir;
      unless ($dry_run) {
         close STDERR;
         open STDERR, ">>$dir/errs.log";
         close STDOUT;
         open STDOUT, ">>$dir/out.log";
      }
      foreach my $seed ( 2002..2006 ) {
         mkdir "$dir/$seed";

         foreach my $instance (@instances) {

            # nq doesn't make sense for general matrices.
            next if $instance =~ m/^M/ && $candidate =~ m/nq/;

            my $out_file = "$dir/$seed/$instance.out";
            my $working_out_file = "$dir/$seed/$instance.working";
            my $instance_with_no_extension = $instance;
            $instance_with_no_extension =~ s/\.tsp$//;

            my $hk = $bound_held_karp{$instance} 
                     || $bound_held_karp{$instance_with_no_extension};

            (defined $hk && $hk > 0) or die "$prog: $instance has no held karp bound";

            my $cmd = "$lk -c $candidate --maxdepth 50 --seed $seed"
                     ." -r two-level -l held_karp $hk -v $instance";

            unless ( -f $out_file ) {
               $cmd .= " >$working_out_file";
               print $cmd,"\n";
               unless ( $dry_run ) {
                  system ($cmd);
                  rename $working_out_file, $out_file
                     or die "$prog: $instance can't rename working file to $out_file: $!";
               }
            }
         }
      }
   }
}
